#version 130

precision mediump float;

uniform vec3 light_position;
uniform vec4 light_color;
uniform vec4 ambient_color;
uniform sampler2D terrainTextureStandard;
uniform sampler2D terrainTextureDirt;
uniform sampler2D shadowMap;
uniform float isShadowOn;

in vec4 frag_color;
in vec4 shadowCoord;
in vec2 texCoord0;
in vec3 world_normal;
in vec4 world_position;

vec4 shadowCoordinate;

out vec4 fragColor;

float linstep(float min, float max, float v)
{
	float x1 = v - min;
	float x2 = max - min;

	float x = x1/x2;

	return clamp(x, 0, 1);
}

float reduceLightBlending(float pmax, float amount)	
{
	return linstep(amount, 1, pmax);
}

float chebyshevUpperBound(float distance)
{
	vec2 moments = texture2D(shadowMap, shadowCoordinate.xy).rg;
	
	if(distance <= moments.x)	{
		return 1.0;
	}

	float variance = moments.y-(moments.x*moments.x);
	variance = max(variance, 0.02);

	float d = distance - moments.x;
	float p_max = variance/(variance+d*d);

	p_max = max(1.0 - p_max, 0.0);

	p_max = reduceLightBlending(p_max, 0.17);

	return p_max + 0.3;
}

void main()
{	
	float shadow = 1.0;

	if(isShadowOn > 0.0)	{
		if(shadowCoord.w > 0.0) {
			shadowCoordinate = shadowCoord/shadowCoord.w;

			shadowCoordinate.z += 0.0005;

			shadow = chebyshevUpperBound(shadowCoordinate.z);
		}
	}

	vec3 normal = normalize(world_normal);

	vec3 position = world_position.xyz/world_position.w;
    
	vec3 light_dir = normalize(light_position - position);

	vec4 ambient = ambient_color * frag_color;

	vec4 diffuse = frag_color * light_color * max(0.0, dot(normal, light_dir));
    
	fragColor = (ambient + diffuse) * shadow;

	fragColor = fragColor * texture(terrainTextureStandard, texCoord0.st);
}
